use std::sync::mpsc::Sender;

use color_eyre::Result;
use ratatui::{
    layout::{Alignment, Constraint, Rect}, 
    text::Text, 
    widgets::{Block, Cell, Row, Table}, 
    Frame
};

use super::{Component, PartialComponent, CurrentTab};
use crate::{action::Action, config::Config, palette::StylePalette};

#[derive(Debug)]
pub struct CurrentValidatorDetails {
    network_tx: Option<Sender<Action>>,
    is_active: bool,
    palette: StylePalette,
    choosen: [u8; 32],
    total_balance: u128,
    own_balance: u128,
    others_len: usize,
    commission: f64,
    points_ratio: f64,
}

impl Default for CurrentValidatorDetails {
    fn default() -> Self {
        Self::new()
    }
}

impl CurrentValidatorDetails {
    const TICKER: &str = " CSPR";
    const DECIMALS: usize = 5;

    pub fn new() -> Self {
        CurrentValidatorDetails {
            network_tx: None,
            is_active: false,
            choosen: [0u8; 32],
            total_balance: 0,
            own_balance: 0,
            others_len: 0,
            commission: 0.0,
            points_ratio: 0.0,
            palette: Default::default(),
        }
    }

    fn prepare_u128(&self, value: u128) -> String {
        let value = value as f64 / 10f64.powi(18);
        let after = Self::DECIMALS;
        format!("{:.after$}{}", value, Self::TICKER)
    }

    fn update_choosen_validator(&mut self, account_id: [u8; 32], individual: u32, total: u32) {
        self.choosen = account_id;
        self.points_ratio = match total {
            0 => 0.0,
            _ => individual as f64 / total as f64,
        };
        if let Some(network_tx) = &self.network_tx {
            let _ = network_tx.send(Action::GetErasStakersOverview(account_id, false));
            let _ = network_tx.send(Action::GetNominatorsByValidator(account_id, false));
            let _ = network_tx.send(Action::GetValidatorPrefs(account_id, false));
        }
    }
}

impl PartialComponent for CurrentValidatorDetails {
    fn set_active(&mut self, _current_tab: CurrentTab) {}
}

impl Component for CurrentValidatorDetails {
    fn register_network_handler(&mut self, tx: Sender<Action>) -> Result<()> {
        self.network_tx = Some(tx);
        Ok(())
    }

    fn register_config_handler(&mut self, config: Config) -> Result<()> {
        if let Some(style) = config.styles.get(&crate::app::Mode::Nominator) {
            self.palette.with_normal_style(style.get("normal_style").copied());
            self.palette.with_hover_style(style.get("hover_style").copied());
            self.palette.with_normal_border_style(style.get("normal_border_style").copied());
            self.palette.with_hover_border_style(style.get("hover_border_style").copied());
            self.palette.with_normal_title_style(style.get("normal_title_style").copied());
            self.palette.with_hover_title_style(style.get("hover_title_style").copied());
            self.palette.with_highlight_style(style.get("highlight_style").copied());
            self.palette.with_scrollbar_style(style.get("scrollbar_style").copied());
        }
        Ok(())
    }

    fn update(&mut self, action: Action) -> Result<Option<Action>> {
        match action {
            Action::SetChoosenValidator(account_id, individual, total) => self.update_choosen_validator(account_id, individual, total),
            Action::SetNominatorsByValidator(noms, account_id) if self.choosen == account_id => self.others_len = noms.len(),
            Action::SetValidatorPrefs(commission, _, account_id) if self.choosen == account_id =>
                self.commission = commission as f64 / 1_000_000_000.0,
            Action::SetStakedRatio(total, own, account_id) if self.choosen == account_id => {
                self.total_balance = total;
                self.own_balance = own;
            },
            _ => {}
        };
        Ok(None)
    }

    fn draw(&mut self, frame: &mut Frame, area: Rect) -> Result<()> {
        let [_, place] = super::validator_details_layout(area);
        let (border_style, border_type) = self.palette.create_border_style(self.is_active);

        let table = Table::new(
            vec![
                Row::new(vec![
                    Cell::from(Text::from("Total staked".to_string()).alignment(Alignment::Left)),
                    Cell::from(Text::from(self.prepare_u128(self.total_balance)).alignment(Alignment::Right)), 
                ]),
                Row::new(vec![
                    Cell::from(Text::from("Own stake".to_string()).alignment(Alignment::Left)),
                    Cell::from(Text::from(self.prepare_u128(self.own_balance)).alignment(Alignment::Right)), 
                ]),
                Row::new(vec![
                    Cell::from(Text::from("Nominators".to_string()).alignment(Alignment::Left)),
                    Cell::from(Text::from(self.others_len.to_string()).alignment(Alignment::Right)), 
                ]),
                Row::new(vec![
                    Cell::from(Text::from("Commission".to_string()).alignment(Alignment::Left)),
                    Cell::from(Text::from(format!("{:.4}%", self.commission)).alignment(Alignment::Right)), 
                ]),
                Row::new(vec![
                    Cell::from(Text::from("Points ratio".to_string()).alignment(Alignment::Left)),
                    Cell::from(Text::from(format!("{:.4}%", self.points_ratio)).alignment(Alignment::Right)), 
                ]),
            ],
            [
                Constraint::Max(12),
                Constraint::Min(0),
            ],
        )
        .column_spacing(1)
        .highlight_style(self.palette.create_highlight_style())
        .block(Block::bordered()
            .border_style(border_style)
            .border_type(border_type)
            .title_alignment(Alignment::Right)
            .title_style(self.palette.create_title_style(false))
            .title("Validator details"));

        frame.render_widget(table, place);
        Ok(())
    }
}
